#!/usr/bin/perl
# 
# peak and saddle preprocessing 
# adapted from Max Berger (https://wiki.openstreetmap.org/wiki/User:Maxbe)

use strict;

my @csv_node_list;   	# 2d array

my @line_arr; 		#array for ONE line from csv file 
my @csv_node_list_arr;

my @peak_arr; 		#Sorted array from maxbe csv [0]=ID [3]=distance => adr:hausenumber [4]= dist => peak_dist
my @saddle_arr; 	#Sorted saddle array with directions of saddles [0]=ID [3]=direction => direction

my $csv_peak_file = "topographic_isolation_viefinderpanoramas.txt";
my $csv_saddle_file = "saddledirection_viefinderpanoramas.100.txt";
my $planet_filtered_file = "peaks_saddles.osm";
my $fileout = "peaks_saddles_processed.osm";

my $id_csv = "";
my $id_xml = "";
my $dist_maxbe = 0;
my $dist_group = "";
my $direction = "";
my $line = "";
my $lon = "";
my $lat = "";

my $dummy_id = "";

my $cnt_peak = 0;
my $cnt_saddle = 0;

my $version = 0;
my $header_string = "";

#read peak_dist list from maxbe, add grouped dist and make array
print "reading maxbe peak csv\n";

open ( fhcsv, "<" , $csv_peak_file ) or die $!;

	while ($line = <fhcsv>) {

		if ( substr ($line, 0, 1) ne "#") {

			chomp($line);
			my @line_arr = split (/;/,$line);
			
			$dist_maxbe = @line_arr[3];
	
			if ($dist_maxbe < 1200) {
				$dist_group = "pd_5";
			}
			elsif (($dist_maxbe >= 1200) and ($dist_maxbe < 5000)) {
				$dist_group = "pd_4";
			}
			elsif (($dist_maxbe >= 5000) and ($dist_maxbe < 9000)) {
				$dist_group = "pd_3";
			}
			elsif (($dist_maxbe >= 9000) and ($dist_maxbe < 25000)) {
				$dist_group = "pd_2";
			}
			elsif (($dist_maxbe >= 25000) and ($dist_maxbe < 100000000)) {
				$dist_group = "pd_1";
			}			
			
			@line_arr[4] = $dist_group;
			
			#write array
			push @csv_node_list_arr, \@line_arr;

			# print "$id_csv \n";
		}
	}

close(fhin);

print "sorting array\n";
my @peak_arr =  sort { $a->[0] <=> $b->[0] } @csv_node_list_arr;

#read saddle directions list from maxbe,
print "reading maxbe saddle directions csv\n";

@csv_node_list_arr = ();
@line_arr = ();

open ( fhcsv, "<" , $csv_saddle_file ) or die $!;

	while ($line = <fhcsv>) {

		if ( substr ($line, 0, 1) ne "#") {

			chomp($line);
			my @line_arr = split (/;/,$line);		

			push @csv_node_list_arr, \@line_arr;

		}
	}

close(fhin);

print "sorting array\n";
my @saddle_arr =  sort { $a->[0] <=> $b->[0] } @csv_node_list_arr;

# testausgabe 
print "writing sorted peaks array\n";
open(fhout, ">","peaks_sorted.csv") or die $!;
for my $row (@peak_arr) {
    print fhout join(",", @{$row}), "\n";
}
close(fhout);

print "writing sorted saddle array\n";
open(fhout, ">","saddle_sorted.csv") or die $!;
for my $row (@saddle_arr) {
    print fhout join(",", @{$row}), "\n";
}
close(fhout);

#open filtered peaks and saddles from planet file, check against @csv_node_list and write result_peaks_saddles.osm

print "writing osm\n";

# open input file
open(XML, "<",$planet_filtered_file) or die $!;
open(fhout, ">",$fileout) or die $!;

my $t;
my $id_peak;
my $id_saddle;

# index arrays for faster search 
my @peakListIndex = ();
foreach $t (@peak_arr) {
	my ($id_peak) = @{$t};
	push( @peakListIndex, $id_peak );
}
my @saddleListIndex = ();
foreach $t (@saddle_arr) {
	my ($id_saddle) = @{$t};
	push( @saddleListIndex, $id_saddle );
}

my %PosPeak;
$PosPeak{ $peakListIndex[$_] } = $_ for 0 .. $#peakListIndex;
my %PosSaddle;
$PosSaddle{ $saddleListIndex[$_] } = $_ for 0 .. $#saddleListIndex;

# parse data
while(my $line = <XML>) {

	# node data
	if($line =~ /^\s*\<node/) {
		my ($id)   = ($line =~ /^\s*\<node id=[\'\"](\d+)[\'\"]/);
		# version++
		#my ($version) = ($line =~ /^\s*\<node.*version=[\'\"](\d+)[\'\"]/);
		#$header_string = "version=\" . $version . \"";
		
		print fhout $line;

		# search in @peak_arr for data
		#my $index = first { $peakListIndex[$_] eq $id } 0 .. $#peakListIndex;
		my $index = $PosPeak{$id};
		if (defined $index) {
			my $peak_row = @peak_arr[$index];
			my ($id_csv, $lon, $lat, $dist_maxbe, $dist_group) = @{$peak_row}; 
				$cnt_peak++;
				#print "Peak   " . $cnt_peak . "   \r";
				#print "Peak $cnt_peak\t$id\t$id_csv\r";
				# print fhout "\t\t<tag k=\"dummy_id_peak\" v=\"$id_csv\" \/>\n";
				print fhout "\t\t<tag k=\"peak_dist\" v=\"$dist_group\" \/>\n";
				# print fhout "\t\t<tag k=\"addr:housenumber\" v=\"$dist_maxbe\" \/>\n";		
				#splice( @peak_arr, $index, 1 );
				#splice( @peakListIndex, $index, 1 );
		#goto SHORTIT;
		}

		# search in @saddle_arr  for data
		#my $index = first { $saddleListIndex[$_] eq $id } 0 .. $#saddleListIndex;
		my $index = $PosSaddle{$id};
		if (defined $index) {
			my $saddle_row = @saddle_arr[$index];
			my ($id_csv, $lon, $lat, $direction) = @{$saddle_row};
				$cnt_saddle++;
				#print "Saddle " . $cnt_saddle . "   \r";
				#print "Saddle $cnt_saddle\t$id\t$id_csv\r";
				print fhout "\t\t<tag k=\"dummy_id_saddle\" v=\"$id_csv\" \/>\n";
				print fhout "\t\t<tag k=\"dir_saddle\" v=\"ds_$direction\" \/>\n";
				#splice( @saddle_arr, $index, 1 );
				#splice( @saddleListIndex, $index, 1 );

		}
	
	SHORTIT:
	
	print "Peak: " . $cnt_peak . "  Saddle: " . $cnt_saddle . "   \r";
	
	}
	
	else { 
		print fhout $line;		
	}

}

# print fhout "</osm>\n";
close(fhout);

print "Found $cnt_peak Peaks and $cnt_saddle Saddles\n";

# ausgabe restliche ids
print "writing remaining peaks array\n";
open(fhout, ">","peaks_remaining.csv") or die $!;
for my $row (@peak_arr) {
    print fhout join(",", @{$row}), "\n";
}
close(fhout);

print "writing remaining saddle array\n";
open(fhout, ">","saddle_remaining.csv") or die $!;
for my $row (@saddle_arr) {
    print fhout join(",", @{$row}), "\n";
}
close(fhout);